@@ -41,7 +41,7 @@ TEL;TYPE=VOICE:412 605 0499
 TEL;TYPE=FAX:412 605 0705
 URL:http://www.example.com
 X-SOMETHING:{r}
-UID:{r}
+UID:{uid}
 END:VCARD'''
 
 TASK_TEMPLATE = u'''BEGIN:VCALENDAR
@@ -54,7 +54,7 @@ LAST-MODIFIED;VALUE=DATE-TIME:20140122T151338Z
 SEQUENCE:2
 SUMMARY:Book: Kowlani - Tödlicher Staub
 X-SOMETHING:{r}
-UID:{r}
+UID:{uid}
 END:VTODO
 END:VCALENDAR'''
 
@@ -64,7 +64,7 @@ DTSTART:19970714T170000Z
 DTEND:19970715T035959Z
 SUMMARY:Bastille Day Party
 X-SOMETHING:{r}
-UID:{r}
+UID:{uid}
 END:VEVENT'''
 
 
@@ -76,7 +76,7 @@ END:VCALENDAR'''
 
 
 SIMPLE_TEMPLATE = u'''BEGIN:FOO
-UID:{r}
+UID:{uid}
 X-SOMETHING:{r}
 HAHA:YES
 END:FOO'''
