@@ -18,19 +18,19 @@ from .. import EVENT_TEMPLATE, TASK_TEMPLATE, VCARD_TEMPLATE, \
     assert_item_equals
 
 
-def format_item(item_template):
+def format_item(item_template, uid=None):
     # assert that special chars are handled correctly.
     r = '{}@vdirsyncer'.format(random.random())
-    return Item(item_template.format(r=r))
+    return Item(item_template.format(r=r, uid=uid or r))
 
 
-class BaseStorageTests(object):
+class StorageTests(object):
     @pytest.fixture
     def get_storage_args(self):
         '''
         Return a function with the following properties:
 
-        :param collection: The collection name to use.
+        :param collection: The name of the collection to create and use.
         '''
         raise NotImplementedError()
 
@@ -44,7 +44,12 @@ class BaseStorageTests(object):
 
     @pytest.fixture
     def get_item(self, item_template):
-        return lambda: format_item(item_template)
+        return lambda **kw: format_item(item_template, **kw)
+
+    @pytest.fixture
+    def requires_collections(self):
+        if not getattr(self, 'supports_collections', True):
+            pytest.skip('This storage does not support collections.')
 
     def test_generic(self, s, get_item):
         items = [get_item() for i in range(1, 10)]
@@ -85,7 +90,7 @@ class BaseStorageTests(object):
         href, etag = s.upload(item)
         assert_item_equals(s.get(href)[0], item)
 
-        new_item = get_item()
+        new_item = get_item(uid=item.uid)
         new_etag = s.update(href, new_item, etag)
         # See https://github.com/untitaker/vdirsyncer/issues/48
         assert isinstance(new_etag, (bytes, text_type))
@@ -143,40 +148,42 @@ class BaseStorageTests(object):
         assert self.storage_class.__name__ in repr(s)
         assert s.instance_name is None
 
-
-class SupportsCollections(object):
-
-    def test_discover(self, get_storage_args, get_item):
+    def test_discover(self, requires_collections, get_storage_args, get_item):
         expected = set()
+        items = {}
 
         for i in range(1, 5):
             # Create collections, but use the "collection" attribute because
             # Radicale requires file extensions in their names.
-            expected.add(
-                self.storage_class(
-                    **get_storage_args(collection='test{}'.format(i))
-                ).collection
-            )
+            collection = 'test{}'.format(i)
+            s = self.storage_class(**get_storage_args(collection=collection))
+            items[s.collection] = [s.upload(get_item())]
+            expected.add(s.collection)
 
         d = self.storage_class.discover(
             **get_storage_args(collection=None))
 
-        actual = set(s.collection for s in d)
+        actual = set(args['collection'] for args in d)
         assert actual >= expected
 
-    def test_discover_collection_arg(self, get_storage_args):
+        for storage_args in d:
+            collection = storage_args['collection']
+            if collection not in expected:
+                continue
+            s = self.storage_class(**storage_args)
+            rv = list(s.list())
+            assert rv == items[collection]
+
+    def test_discover_collection_arg(self, requires_collections,
+                                     get_storage_args):
         args = get_storage_args(collection='test2')
         with pytest.raises(TypeError) as excinfo:
             list(self.storage_class.discover(**args))
 
         assert 'collection argument must not be given' in str(excinfo.value)
 
-    def test_collection_arg(self, get_storage_args):
+    def test_collection_arg(self, requires_collections, get_storage_args):
         s = self.storage_class(**get_storage_args(collection='test2'))
         # Can't do stronger assertion because of radicale, which needs a
         # fileextension to guess the collection type.
         assert 'test2' in s.collection
-
-
-class StorageTests(BaseStorageTests, SupportsCollections):
-    pass
