@@ -110,13 +110,16 @@ class ServerMixin(object):
             url = 'http://127.0.0.1/bob/'
             if collection is not None:
                 collection += self.storage_class.fileext
+                url = url.rstrip('/') + '/' + collection
 
             rv = {'url': url, 'username': 'bob', 'password': 'bob',
                   'collection': collection, 'unsafe_href_chars': ''}
 
             if collection is not None:
+                # XXX: Create collection, works around
+                # https://github.com/Kozea/Radicale/pull/236
                 s = self.storage_class(**rv)
-                s.delete(*s.upload(get_item()))  # create collection
+                s.delete(*s.upload(get_item()))
 
             return rv
         return inner
