@@ -12,12 +12,13 @@ import pytest
 
 from vdirsyncer.storage.memory import MemoryStorage
 
-from . import BaseStorageTests
+from . import StorageTests
 
 
-class TestMemoryStorage(BaseStorageTests):
+class TestMemoryStorage(StorageTests):
 
     storage_class = MemoryStorage
+    supports_collections = False
 
     @pytest.fixture
     def get_storage_args(self):
