@@ -7,11 +7,13 @@
     :license: MIT, see LICENSE for more details.
 '''
 import datetime
+import functools
 import itertools
 
 from lxml import etree
 
 from requests import session as requests_session
+from requests.exceptions import RequestException
 
 from .base import Item, Storage
 from .http import USERAGENT, prepare_auth, prepare_verify
@@ -49,10 +51,42 @@ def _parse_xml(content):
                          'Double-check the URLs in your config.'.format(e))
 
 
+def _fuzzy_matches_mimetype(strict, weak):
+    # different servers give different getcontenttypes:
+    # "text/vcard", "text/x-vcard", "text/x-vcard; charset=utf-8",
+    # "text/directory;profile=vCard", "text/directory",
+    # "text/vcard; charset=utf-8"
+    if strict is None or weak is None:
+        return True
+
+    mediatype, subtype = strict.split('/')
+    if subtype in weak:
+        return True
+    return False
+
+
+def _catch_generator_exceptions(f):
+    @functools.wraps(f)
+    def inner(*args, **kwargs):
+        try:
+            for x in f(*args, **kwargs):
+                yield x
+        except (RequestException, exceptions.Error):
+            pass
+    return inner
+
+
 class Discover(object):
+    # Another one of Radicale's specialties: Discovery is broken (returning
+    # completely wrong URLs at every stage) as of version 0.9.
+    # https://github.com/Kozea/Radicale/issues/196
+    #
+    # So we just brute-force a lot of paths here.
+
     _resourcetype = None
     _homeset_xml = None
     _homeset_tag = None
+    _well_known_uri = None
     _collection_xml = """
     <d:propfind xmlns:d="DAV:">
         <d:prop>
@@ -65,7 +99,13 @@ class Discover(object):
     def __init__(self, session):
         self.session = session
 
-    def _find_principal(self):
+    def find_principal(self):
+        for server in self.find_dav():
+            for x in list(self._find_principal(server)) or ['']:
+                yield x
+
+    @_catch_generator_exceptions
+    def _find_principal(self, url):
         """tries to find the principal URL of the user
         :returns: iterable (but should be only of element) of urls
         :rtype: iterable(unicode)
@@ -80,8 +120,8 @@ class Discover(object):
             </d:prop>
         </d:propfind>
         """
-        response = self.session.request('PROPFIND', '', headers=headers,
-                                        data=body)
+        response = self.session.request('PROPFIND', url, headers=headers,
+                                        data=body, is_subpath=False)
         root = _parse_xml(response.content)
 
         for element in root.iter('{*}current-user-principal'):
@@ -89,26 +129,37 @@ class Discover(object):
                 if principal.tag.endswith('href'):
                     yield principal.text
 
+    def find_dav(self):
+        return list(self._find_dav()) or ['']
+
+    @_catch_generator_exceptions
+    def _find_dav(self):
+        response = self.session.request('GET', self._well_known_uri,
+                                        allow_redirects=False,
+                                        is_subpath=False)
+        yield response.headers.get('Location', '')
+
     def discover(self):
         """discover all the user's CalDAV or CardDAV collections on the server
         :returns: a list of the user's collections (as urls)
         :rtype: list(unicode)
         """
-        # Another one of Radicale's specialties: Discovery is broken (returning
-        # completely wrong URLs at every stage) as of version 0.9.
-        # https://github.com/Kozea/Radicale/issues/196
-        done = set()
-        for principal in itertools.chain(self._find_principal(), ['']):
-            for home in itertools.chain(self._find_home(principal), ['']):
-                for collection in self._find_collections(home):
-                    collection['href'] = href = \
-                        utils.urlparse.urljoin(self.session.url,
-                                               collection['href'])
-                    if href not in done:
-                        done.add(href)
-                        yield collection
 
-    def _find_home(self, principal):
+        done = set()
+        for collection in self.find_collections():
+            collection['href'] = href = \
+                utils.urlparse.urljoin(self.session.url, collection['href'])
+            if href not in done:
+                done.add(href)
+                yield collection
+
+    def find_homes(self):
+        for principal in self.find_principal():
+            for x in self._find_homes(principal):
+                yield x
+
+    @_catch_generator_exceptions
+    def _find_homes(self, principal):
         headers = self.session.get_default_headers()
         headers['Depth'] = 0
         response = self.session.request('PROPFIND', principal, headers=headers,
@@ -121,6 +172,12 @@ class Discover(object):
                 if homeset.tag.endswith('href'):
                     yield homeset.text
 
+    def find_collections(self):
+        for home in itertools.chain(self.find_homes(), ['']):
+            for x in self._find_collections(home):
+                yield x
+
+    @_catch_generator_exceptions
     def _find_collections(self, home):
         """find all CalDAV collections under `home`"""
 
@@ -154,6 +211,7 @@ class CalDiscover(Discover):
     </d:propfind>
     """
     _homeset_tag = '{*}calendar-home-set'
+    _well_known_uri = '/.well-known/caldav/'
 
 
 class CardDiscover(Discover):
@@ -166,6 +224,7 @@ class CardDiscover(Discover):
     </d:propfind>
     """
     _homeset_tag = '{*}addressbook-home-set'
+    _well_known_uri = '/.well-known/carddav/'
 
 
 class DavSession(object):
@@ -188,16 +247,17 @@ class DavSession(object):
         self.parsed_url = utils.urlparse.urlparse(self.url)
         self._session = None
 
-    def request(self, method, path, data=None, headers=None,
-                is_subpath=True):
+    def request(self, method, path, is_subpath=True, **kwargs):
         url = self.url
         if path:
             url = utils.urlparse.urljoin(self.url, path)
         assert url.startswith(self.url) or not is_subpath
         if self._session is None:
             self._session = requests_session()
-        return utils.request(method, url, data=data, headers=headers,
-                             session=self._session, **self._settings)
+
+        more = dict(self._settings)
+        more.update(kwargs)
+        return utils.request(method, url, session=self._session, **more)
 
     def get_default_headers(self):
         return {
@@ -209,12 +269,6 @@ class DavSession(object):
 class DavStorage(Storage):
 
     '''
-    .. note::
-
-        Please also see :doc:`supported` for very important information, as
-        changing some of the default options might be very dangerous with some
-        servers.
-
     :param url: Base URL or an URL to a collection.
     :param username: Username for authentication.
     :param password: Password for authentication.
@@ -228,6 +282,12 @@ class DavStorage(Storage):
     :param useragent: Default ``vdirsyncer``.
     :param unsafe_href_chars: Replace the given characters when generating
         hrefs. Defaults to ``'@'``.
+
+    .. note::
+
+        Please also see :doc:`supported` for very important information, as
+        changing some of the default options might be very dangerous with some
+        servers.
     '''
 
     # the file extension of items. Useful for testing against radicale.
@@ -250,9 +310,6 @@ class DavStorage(Storage):
         super(DavStorage, self).__init__(**kwargs)
 
         url = url.rstrip('/') + '/'
-        collection = kwargs.get('collection')
-        if collection is not None:
-            url = utils.urlparse.urljoin(url, collection)
         self.session = DavSession(url, username, password, verify, auth,
                                   useragent, verify_fingerprint)
         self.unsafe_href_chars = unsafe_href_chars
@@ -262,19 +319,53 @@ class DavStorage(Storage):
         self.url = url
 
     @classmethod
-    def discover(cls, url, **kwargs):
-        if kwargs.pop('collection', None) is not None:
-            raise TypeError('collection argument must not be given.')
+    def _get_session(cls, **kwargs):
         discover_args, _ = utils.split_dict(kwargs, lambda key: key in (
-            'username', 'password', 'verify', 'auth', 'useragent',
+            'url', 'username', 'password', 'verify', 'auth', 'useragent',
             'verify_fingerprint',
         ))
-        d = cls.discovery_class(DavSession(url=url, **discover_args))
+        return DavSession(**discover_args)
+
+    @classmethod
+    def discover(cls, **kwargs):
+        if kwargs.pop('collection', None) is not None:
+            raise TypeError('collection argument must not be given.')
+
+        d = cls.discovery_class(cls._get_session(**kwargs))
         for c in d.discover():
-            base, collection = c['href'].rstrip('/').rsplit('/', 1)
-            s = cls(url=base, collection=collection, **kwargs)
-            s.displayname = c['displayname']
-            yield s
+            url = c['href']
+            _, collection = url.rstrip('/').rsplit('/', 1)
+            storage_args = dict(kwargs)
+            storage_args.update({'url': url, 'collection': collection,
+                                 'collection_human': c['displayname']})
+            yield storage_args
+
+    @classmethod
+    def join_collection(cls, collection, **kwargs):
+        session = cls._get_session(**kwargs)
+        d = cls.discovery_class(session)
+
+        for c in cls.discover(**kwargs):
+            if c['collection'] == collection:
+                return c
+
+        homes = list(d.find_homes())
+        if len(homes) != 1:
+            raise NotImplementedError('Not sure where to create {r!}, {} '
+                                      'homeset-URLs found (need exactly 1).'
+                                      .format(collection, len(homes)))
+
+        try:
+            collection_url = '{}/{}'.format(homes[0].rstrip('/'), collection)
+            response = d.session.request('MKCOL', collection_url,
+                                         is_subpath=False)
+        except RequestException as e:
+            raise NotImplementedError(e)
+        else:
+            rv = dict(kwargs)
+            rv['collection'] = collection
+            rv['url'] = response.url
+            return rv
 
     def _normalize_href(self, *args, **kwargs):
         return _normalize_href(self.session.url, *args, **kwargs)
@@ -283,7 +374,10 @@ class DavStorage(Storage):
         href = item.ident
         for char in self.unsafe_href_chars:
             href = href.replace(char, '_')
-        return self._normalize_href(href + self.fileext)
+        return _encode_href(self._normalize_href(href + self.fileext))
+
+    def _is_item_mimetype(self, mimetype):
+        return _fuzzy_matches_mimetype(self.item_mimetype, mimetype)
 
     def get(self, href):
         ((actual_href, item, etag),) = self.get_multi([href])
@@ -404,8 +498,6 @@ class DavStorage(Storage):
                 dav_logger.warning('Skipping identical href: {!r}'
                                    .format(href))
                 continue
-            else:
-                hrefs.add(href)
 
             try:
                 prop = response.find('{DAV:}propstat/{DAV:}prop')
@@ -421,25 +513,21 @@ class DavStorage(Storage):
                 dav_logger.debug('Skipping {!r}, is collection.'.format(href))
                 continue
 
-            # different servers give different getcontenttypes:
-            # "text/vcard", "text/x-vcard", "text/x-vcard; charset=utf-8",
-            # "text/directory;profile=vCard", "text/directory",
-            # "text/vcard; charset=utf-8"
-            if self.item_mimetype is not None:
-                mediatype, subtype = self.item_mimetype.split('/')
-                if contenttype is not None and subtype not in contenttype.text:
-                    dav_logger.debug('Skipping {!r}, {!r} != {!r}.'
-                                     .format(href, contenttype,
-                                             self.item_mimetype))
-                    continue
+            contenttype = getattr(contenttype, 'text', None)
+            if not self._is_item_mimetype(contenttype):
+                dav_logger.debug('Skipping {!r}, {!r} != {!r}.'
+                                 .format(href, contenttype,
+                                         self.item_mimetype))
+                continue
 
+            hrefs.add(href)
             yield href, etag, prop
 
 
 class CaldavStorage(DavStorage):
 
     __doc__ = '''
-    CalDAV. Usable as ``caldav`` in the config file.
+    CalDAV.
 
     You can set a timerange to synchronize with the parameters ``start_date``
     and ``end_date``. Inside those parameters, you can use any Python
@@ -453,15 +541,12 @@ class CaldavStorage(DavStorage):
     Either both or none have to be specified. The default is to synchronize
     everything.
 
-    ''' + DavStorage.__doc__ + '''
     :param start_date: Start date of timerange to show, default -inf.
     :param end_date: End date of timerange to show, default +inf.
     :param item_types: Comma-separated collection types to show from the
         server. Dependent on server functionality, no clientside validation of
-        results. The empty value ``''`` is the same as ``'VTODO, VEVENT,
-        VJOURNAL'``.
-
-    '''
+        results. The empty value ``[]`` means "all items".
+    ''' + DavStorage.__doc__
 
     storage_name = 'caldav'
     fileext = '.ics'
@@ -486,18 +571,8 @@ class CaldavStorage(DavStorage):
     def __init__(self, start_date=None, end_date=None,
                  item_types=('VTODO', 'VEVENT'), **kwargs):
         super(CaldavStorage, self).__init__(**kwargs)
-        if isinstance(item_types, str):
-            orig_item_types = item_types
-            item_types = list(filter(
-                bool, (x.strip() for x in item_types.split(','))
-            ))
-
-            # XXX: Deprecation
-            import json
-            dav_logger.warning(
-                '{!r} is deprecated, please use:\nitem_types = {}\n'
-                'The old form will be removed in 0.4.0.'
-                .format(orig_item_types, json.dumps(item_types)))
+        if not isinstance(item_types, (list, tuple)):
+            raise ValueError('item_types must be a list.')
 
         self.item_types = tuple(item_types)
         if (start_date is None) != (end_date is None):
@@ -586,7 +661,7 @@ class CaldavStorage(DavStorage):
 class CarddavStorage(DavStorage):
 
     __doc__ = '''
-    CardDAV. Usable as ``carddav`` in the config file.
+    CardDAV.
     ''' + DavStorage.__doc__
 
     storage_name = 'carddav'
