@@ -20,9 +20,10 @@ logger = log.get(__name__)
 class FilesystemStorage(Storage):
 
     '''
-    Saves each item in its own file, given a directory. Can be used with `khal
-    <http://lostpackets.de/khal/>`_. See :doc:`vdir` for a more formal
-    description of the format. Usable as ``filesystem`` in the config file.
+    Saves each item in its own file, given a directory.
+
+    Can be used with `khal <http://lostpackets.de/khal/>`_. See :doc:`vdir` for
+    a more formal description of the format.
 
     :param path: Absolute path to a vdir or collection, depending on the
         collection parameter (see :py:class:`vdirsyncer.storage.base.Storage`).
@@ -40,9 +41,6 @@ class FilesystemStorage(Storage):
     def __init__(self, path, fileext, encoding='utf-8', create=True, **kwargs):
         super(FilesystemStorage, self).__init__(**kwargs)
         path = expand_path(path)
-        collection = kwargs.get('collection')
-        if collection is not None:
-            path = os.path.join(path, collection)
         checkdir(path, create=create)
         self.path = path
         self.encoding = encoding
@@ -53,10 +51,23 @@ class FilesystemStorage(Storage):
         if kwargs.pop('collection', None) is not None:
             raise TypeError('collection argument must not be given.')
         path = expand_path(path)
-        for collection in os.listdir(path):
-            if os.path.isdir(os.path.join(path, collection)):
-                s = cls(path=path, collection=collection, **kwargs)
-                yield s
+        try:
+            collections = os.listdir(path)
+        except OSError:
+            if not kwargs.get('create', True):
+                raise
+        else:
+            for collection in collections:
+                collection_path = os.path.join(path, collection)
+                if os.path.isdir(collection_path):
+                    args = dict(collection=collection, path=collection_path,
+                                **kwargs)
+                    yield args
+
+    @classmethod
+    def join_collection(cls, collection, **kwargs):
+        kwargs['path'] = os.path.join(kwargs['path'], collection)
+        return kwargs
 
     def _get_filepath(self, href):
         return os.path.join(self.path, href)
